@@ -4,13 +4,8 @@
 Draupnir : Ancestral protein sequence reconstruction using a tree-structured Ornstein-Uhlenbeck variational autoencoder
 =======================
 """
-from .main import *
-from .datasets import *
-from .utils import *
-from .draw_tree import *
-from .mutual_information import *
-#from .models_utils import *
-#from .guides import *
-#from .models import *
-#from .train import *
+from .Draupnir_main import *
+from .Draupnir_datasets import *
+from .Draupnir_utils import *
+
 
